﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.RoutePlanning {
  [Item("Route Planning Problem", "Represents a Route Planning Problem.")]
  [Creatable("Problems")]
  [StorableClass]
  public class RoutePlanningProblem : Problem {
    private const string CoordinatesParameterName = "Coordinates";
    private const string FromParameterName = "From";
    private const string ToParameterName = "To";
    public string Filename { get; set; }

    #region Parameter Properties

    public OptionalValueParameter<DoubleMatrix> CoordinatesParameter {
      get { return (OptionalValueParameter<DoubleMatrix>)Parameters[CoordinatesParameterName]; }
    }
    public ValueParameter<IntValue> FromParameter {
      get { return (ValueParameter<IntValue>)Parameters[FromParameterName]; }
    }
    public ValueParameter<IntValue> ToParameter {
      get { return (ValueParameter<IntValue>)Parameters[ToParameterName]; }
    }

    #endregion

    #region Properties

    public DoubleMatrix Coordinates {
      get { return CoordinatesParameter.Value; }
      set { CoordinatesParameter.Value = value; }
    }

    public IntValue From {
      get { return FromParameter.Value; }
      set { FromParameter.Value = value; }
    }

    public IntValue To {
      get { return ToParameter.Value; }
      set { ToParameter.Value = value; }
    }

    #endregion

    #region Constructors

    [StorableConstructor]
    private RoutePlanningProblem(bool deserializing) : base(deserializing) { }

    private RoutePlanningProblem(RoutePlanningProblem original, Cloner cloner)
      : base(original, cloner) {
      RegisterEventHandlers();
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new RoutePlanningProblem(this, cloner);
    }

    public RoutePlanningProblem() {
      Parameters.Add(new OptionalValueParameter<DoubleMatrix>(CoordinatesParameterName, "The x- and y-Coordinates of the cities."));
      Parameters.Add(new ValueParameter<IntValue>(FromParameterName, "From."));
      Parameters.Add(new ValueParameter<IntValue>(ToParameterName, "To."));

      RegisterEventHandlers();
    }

    #endregion

    private void RegisterEventHandlers() { }

  }
}
